# glam2csv.pl for Perl v4 and v5 3-Sep-02  John Dudeck
# Buspkg conversion routine for Account Master file
# convert GLAMAS.DDF to .csv for import to Excel
# To use, change directory to the folder with GLAMAS.DDF and type 'perl (path...)\glamas2csv.pl'
# Input is GLAMAS.DDF created by ISLOAD dump of GLAMAS isam file
# Output is GLAMAS.CSV

#routine to trim trailing spaces
sub trim {@word = @_; $word[0] =~ s/\s+$//; return $word[0]; }

#routine to handle DBL's negative number encoding
sub convdbl {
 $anum = @_[0];
 $dig = substr($anum,-1,1); 		# pull off last digit
 $neg = ($dig =~ /[p-y]/) ? -1 : 1; # last digit has hex 40 bit set if number is negative
 $dig =~ tr/pqrstuvwxy/0123456789/ ;# clear the hex 40 bit on last digit
 substr($anum,-1,1) = $dig;			# put the digit back
 if ($anum == 0) {$neg = 1;}		# deal with negative zeros
 return $anum * $neg; 				# convert string to numeric by multiplication
}

# check command line arguments -- $argu will contian -c if there should be no cents in formatting
$argu = shift(@ARGV);
if ($argu && $argu ne '-c') {
 die "usage: $0 [-h] [-c]\n  -c format currencies with no cents\n  -h print this help\n";
}

open (INFILE, "glamas.ddf") || die "glamas.ddf not found";
open (OUTFILE, ">glamas.csv") || die "cannot create glamas.csv";
# Print the column headers for import.
print OUTFILE "\"Company\",\"Major\",\"Minor\",\"Description\",\"FSCODE\",\"SCHCDE1\",\"SCHCDE2\",\"SCHCDE3\",\"SCHCDE4\",\"Paren\",";
print OUTFILE "\"Bud1\",\"Bud2\",\"Bud3\",\"Bud4\",\"Bud5\",\"Bud6\",\"Bud7\",\"Bud8\",\"Bud9\",\"Bud10\",\"Bud11\",\"Bud12\",\"Bud13\",";
print OUTFILE "\"PrYr0\",\"PrYr1\",\"PrYr2\",\"PrYr3\",\"PrYr4\",\"PrYr5\",\"PrYr6\",\"PrYr7\",\"PrYr8\",\"PrYr9\",\"PrYr10\",\"PrYr11\",\"PrYr12\",\"PrYr13\",";
print OUTFILE "\"Dept\"\n";
# read thru file and convert each record
while ($line = <INFILE>) {
 #Company(2), Major(6), Minor(6), Description(30),
 print OUTFILE "\"", substr($line,0,2), "\",\"", substr($line,2,6), "\",\"", substr($line,8,6), "\",\"", &trim(substr($line,14,30)), "\"," ;
 #Fin Stmt Code(6) & four Schedule codes(7 ea)
 print OUTFILE "\"", substr($line,44,6), "\",\"", &trim(substr($line,50,7)), "\",\"", &trim(substr($line,57,7)), "\",\"", &trim(substr($line,64,7)), "\",\"", &trim(substr($line,71,7)), "\",";
 #Paren(1)
 print OUTFILE "\"", substr($line,78,1), "\",";
 # 13 Budgets (9 dig ea)
 for ($i=0; $i<13; $i++) {
  $num = &convdbl(substr($line, 79 + $i * 9, 9));
  if ($argu) {					# true if no cents
   $num = ($num * 100);			# add 2 zeros
  }
  print OUTFILE $num, ",";
 }
 #14 Prior Year (11 dig ea)
 for ($i=0; $i<14; $i++) {
  $num = &convdbl(substr($line, 196 + $i * 11, 11));
  if ($argu) {						# true if no cents
   $num = $num."";					# just convert to string
  } else {
   $num = sprintf("%.2f", $num/100);	# scale by 100 return formatted decimal
  }
  print OUTFILE $num, ",";
 }
 #Dept(2)
 print OUTFILE "\"", &trim(substr($line,350,2)), "\"";
 print OUTFILE "\n";
}
close INFILE;
close OUTFILE;



