# ytd2csv.pl for Perl v4 and v5 3-Sep-02  John Dudeck
# Buspkg conversion routine for year to date transaction file
# convert YTDGLT.DDF to .csv for import to Excel
# To use, change directory to the folder with YTDGLT.DDF and type 'perl (path...)\ytdglt2csv.pl'
# Input is YTDGLT.DDF created by ISLOAD dump of YTDGLT isam file
# Output is YTDGLT.CSV

#routine to trim trailing spaces
sub trim { @word = @_; $word[0] =~ s/\s+$//; return $word[0]; }

#routine to handle DBL's negative number encoding
sub convdbl {
 $anum = @_[0];
 $dig = substr($anum,-1,1); 		# pull off last digit
 $neg = ($dig =~ /[p-y]/) ? -1 : 1; # last digit has hex 40 bit set if number is negative
 $dig =~ tr/pqrstuvwxy/0123456789/ ;# clear the hex 40 bit on last digit
 substr($anum,-1,1) = $dig;			# put the digit back
 if ($anum == 0) {$neg = 1;}		# deal with negative zeros
 return $anum * $neg; 				# convert string to numeric by multiplication
}

# check command line arguments -- $argu will contian -c if there should be no cents in formatting
$argu = shift(@ARGV);
if ($argu && $argu ne '-c') {
 die "usage: $0 [-h] [-c]\n  -c format currencies with no cents\n  -h print this help\n";
}

open (INFILE, "ytdglt.ddf") || die "ytdglt.ddf not found";
open (OUTFILE, ">ytdglt.csv") || die "cannot create ytdglt.csv";
# Print the column headers for import.
print OUTFILE "\"Company\",\"Major\",\"Minor\",\"Date\",\"Amount\",\"Source\",\"Description\"\n";
# read thru file and convert each record
while ($line = <INFILE>) {
 #Company(2), Major(6), Minor(6)
 print OUTFILE "\"", substr($line,0,2), "\",\"", substr($line,2,6), "\",\"", substr($line,8,6), "\",";
 #Date(6)
 print OUTFILE substr($line,14,2), "/", substr($line,16,2), "/", substr($line,18,2), ",";
 #Amount(11)
 $num = &convdbl(substr($line,20,10));
 if ($argu) {						# true if no cents
  $num = $num."";					# just convert to string
 } else {
  $num = sprintf("%.2f", $num/100);	# scale by 100 return formatted decimal
 }
 print OUTFILE $num, ",";
 #Source Code (5)
 print OUTFILE "\"", &trim(substr($line,30,5)), "\",";
 #Description(30)
 print OUTFILE "\"", &trim(substr($line,35,30)), "\"";
 print OUTFILE "\n";
}
close INFILE;
close OUTFILE;



